from lkb.factory import lkb_connect
from components.entity_linkers.ngram_no_context_linker.ngram_text_linker import (
    LINKER_PATHS,
    ngramEntityLinker,
)


class APIController:
    def __init__(self, lkb, no_context_entity_linker):
        self._lkb = lkb
        self._ncel = no_context_entity_linker

    def get_concept_info(self, concept_id):

        concept = self._lkb.get_concept(concept_id)
        terms = [
            {"term_id": term.id, "term_string": term.string}
            for term in self._lkb.get_terms_from_concept_id(concept_id)
        ]
        # Bring "isa" to the front

        inward_edges = self._lkb.get_inward_edges(concept_id)
        inverse_isa_edges = filter(
            lambda edge: edge[0].string == "inverse_isa", inward_edges
        )
        non_inverse_isa_edges = filter(
            lambda edge: edge[0].string != "inverse_isa", inward_edges
        )
        inward_related_concepts = [
            {
                "relation_string": relation.string,
                "concept_id": concept.id,
                "concept_name": concept.concept_name,
            }
            for relation, concept in list(inverse_isa_edges)
            + list(non_inverse_isa_edges)
        ]
        # Bring "inverse_isa" to the front
        outward_edges = self._lkb.get_outward_edges(concept_id)
        isa_edges = filter(lambda edge: edge[0].string == "isa", outward_edges)
        non_isa_edges = filter(lambda edge: edge[0].string != "isa", outward_edges)
        outward_related_concepts = [
            {
                "relation_string": relation.string,
                "concept_id": concept.id,
                "concept_name": concept.concept_name,
            }
            for relation, concept in list(isa_edges) + list(non_isa_edges)
        ]

        return {
            "id": concept.id,
            "concept_name": concept.concept_name,
            "semantic_type": concept.semantic_type,
            "definition": concept.definition,
            "terms": terms,
            "inward_related_concepts": inward_related_concepts,
            "outward_related_concepts": outward_related_concepts,
        }

    def get_term_info(self, term_id):
        term = self._lkb.get_term(term_id)
        concepts = self._lkb.get_concepts_from_term_id(term_id)
        return {
            "term_id": term.id,
            "term_string": term.string,
            "concepts": [
                {"concept_id": concept.id, "concept_name": concept.concept_name}
                for concept in concepts
            ],
        }

    def search_for_concept(self, search_term):
        concepts = [
            self._lkb.get_concept(concept_id)
            for concept_id in self._ncel.link_texts([search_term])[0]
        ]
        return [
            {"concept_id": concept.id, "concept_name": concept.concept_name}
            for concept in concepts
        ]


def create_trendnet_api_controller(trendnet_name):
    trendnet_linker_path = LINKER_PATHS[trendnet_name]
    trendnet = lkb_connect(trendnet_linker_path.lkb_name)
    no_context_entity_linker = ngramEntityLinker.load(
        linker_paths=trendnet_linker_path.linker_components_path, lkb=trendnet
    )
    return APIController(trendnet, no_context_entity_linker)


# print me
